@@ -15,7 +15,7 @@ our @EXPORT_OK = (
 );
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v6.17.0";
+our $VERSION = "v6.18.0";
 
 1;
 __END__
@@ -61,7 +61,7 @@ The documentation in this POD applies to all three adapter modules as well.
 This module is an implementation of L<Reactive Extensions|http://reactivex.io/> in Perl. It replicates the
 behavior of L<rxjs 6|https://www.npmjs.com/package/rxjs> which is the JavaScript implementation of ReactiveX.
 
-Currently 73 of the 100+ operators in rxjs are implemented in this module.
+Currently 77 of the 100+ operators in rxjs are implemented in this module.
 
 =head1 EXPORTABLE FUNCTIONS
 
@@ -459,14 +459,25 @@ Similar to rx_combine_latest, but as a pipeable operator.
         op_take(10),
     )->subscribe($observer);
 
+=item op_concat_all
+
+L<https://rxjs.dev/api/operators/concatAll>
+
+    # 0, 1, 2, 0, 1, 2, 0, 1, complete
+    rx_interval(0.7)->pipe(
+        op_map(sub { rx_interval(1)->pipe( op_take(3) ) }),
+        op_concat_all(),
+        op_take(10),
+    )->subscribe($observer);
+
 =item op_concat_map
 
 L<https://rxjs.dev/api/operators/concatMap>
 
     # 0, 1, 2, 0, 1, 2, 0, 1, 2, complete
     rx_of(10, 20, 30)->pipe(
-        op_concat_map(sub {
-            rx_interval(1)->pipe(op_take(3)),
+        op_concat_map(sub ($val, $idx) {
+            rx_interval(1)->pipe(op_take(3)), # can also use $_ here instead of $val
         }),
     )->subscribe($observer);
 
@@ -604,14 +615,25 @@ L<https://rxjs.dev/api/operators/every>
         op_every(sub ($value, $idx) { $value % 5 == 0 }),
     )->subscribe($observer);
 
+=item op_exhaust_all
+
+L<https://rxjs.dev/api/operators/exhaustAll>
+
+    # 0, 1, 2, 3, 0, 1, 2, 3, complete
+    rx_interval(3)->pipe(
+        op_take(3),
+        op_map(sub { rx_interval(1)->pipe( op_take(4) ) }),
+        op_exhaust_all(),
+    )->subscribe($observer);
+
 =item op_exhaust_map
 
 L<https://rxjs.dev/api/operators/exhaustMap>
 
     # 0, 1, 2, complete
     rx_of(10, 20, 30)->pipe(
-        op_exhaust_map(sub {
-            rx_interval(1)->pipe( op_take(3) );
+        op_exhaust_map(sub ($val, $idx) {
+            rx_interval(1)->pipe( op_take(3) ); # can also use $_ here instead of $val
         }),
     )->subscribe($observer);
 
@@ -720,15 +742,25 @@ L<https://rxjs.dev/api/operators/mapTo>
         op_map_to(123),
     )->subscribe($observer);
 
+=item op_merge_all
+
+L<https://rxjs.dev/api/operators/mergeAll>
+
+    # 0, 1, 0, 2, 1, 3, 2, 0, 3, 1, 0, ...
+    rx_interval(1)->pipe(
+        op_map(sub { rx_interval(0.7)->pipe(op_take(4)) }),
+        op_merge_all(2),
+    )->subscribe($observer);
+
 =item op_merge_map
 
 L<https://rxjs.dev/api/operators/mergeMap>
 
     # 11, 21, 31, 12, 22, 32, 13, 23, 33, complete
     rx_of(10, 20, 30)->pipe(
-        op_merge_map(sub ($x) {
+        op_merge_map(sub ($x, $idx) {
             return rx_interval(1)->pipe(
-                op_map(sub ($y) {
+                op_map(sub ($y, @) {
                     return $x + $y + 1;
                 }),
                 op_take(3),
@@ -894,6 +926,17 @@ L<https://rxjs.dev/api/operators/startWith>
         op_start_with(100, 200),
     )->subscribe($observer);
 
+=item op_switch_all
+
+L<https://rxjs.dev/api/operators/switchAll>
+
+    # 0, 0, 0, 1, 2, 3, 4, complete
+    rx_timer(0, 3)->pipe(
+        op_take(3),
+        op_map(sub { rx_interval(2)->pipe(op_take(5)) }),
+        op_switch_all(),
+    )->subscribe($observer);
+
 =item op_switch_map
 
 L<https://rxjs.dev/api/operators/switchMap>
@@ -902,9 +945,9 @@ L<https://rxjs.dev/api/operators/switchMap>
     my $o = rx_interval(2.5)->pipe( op_take(3) );
 
     $o->pipe(
-        op_switch_map(sub ($x) {
+        op_switch_map(sub ($x, $idx) {
             return rx_interval(0.7)->pipe(
-                op_map(sub ($y) { $x * 10 + $y + 1 }),
+                op_map(sub ($y, $idx2) { $x * 10 + $y + 1 }),
             );
         }),
     )->subscribe($observer);
