@@ -15,17 +15,17 @@ use Scalar::Util 'reftype', 'refaddr', 'blessed', 'weaken';
 
 use Exporter 'import';
 our @EXPORT_OK = qw/
-    op_audit_time op_buffer op_buffer_count op_buffer_time op_catch_error op_combine_latest_with op_concat_map
-    op_concat_with op_count op_debounce_time op_default_if_empty op_delay op_distinct_until_changed
-    op_distinct_until_key_changed op_element_at op_end_with op_every op_exhaust_map op_filter op_finalize op_first
-    op_ignore_elements op_map op_map_to op_merge_map op_merge_with op_multicast op_on_error_resume_next_with op_pairwise
-    op_pluck op_reduce op_ref_count op_repeat op_retry op_sample_time op_scan op_share op_skip op_skip_until
-    op_skip_while op_start_with op_switch_map op_take op_take_until op_take_while op_tap op_throttle_time
-    op_with_latest_from op_zip_with
+    op_audit_time op_buffer op_buffer_count op_buffer_time op_catch_error op_combine_latest_with op_concat_all
+    op_concat_map op_concat_with op_count op_debounce_time op_default_if_empty op_delay op_distinct_until_changed
+    op_distinct_until_key_changed op_element_at op_end_with op_every op_exhaust_all op_exhaust_map op_filter
+    op_finalize op_first op_ignore_elements op_map op_map_to op_merge_all op_merge_map op_merge_with op_multicast
+    op_on_error_resume_next_with op_pairwise op_pluck op_reduce op_ref_count op_repeat op_retry op_sample_time op_scan
+    op_share op_skip op_skip_until op_skip_while op_start_with op_switch_all op_switch_map op_take op_take_until
+    op_take_while op_tap op_throttle_time op_with_latest_from op_zip_with
 /;
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v6.17.0";
+our $VERSION = "v6.18.0";
 
 sub op_audit_time {
     my ($duration) = @_;
@@ -251,39 +251,12 @@ sub op_combine_latest_with {
     }
 }
 
-sub _op_concat_map_helper {
-    my (
-        $value, $inner_subscriptions, $observable_factory,
-        $subscriber, $queue, $own_subscription
-    ) = @_;
+sub op_concat_all {
+    return sub {
+        my ($source) = @_;
 
-    if (@$inner_subscriptions and ! $inner_subscriptions->[0]{closed}) {
-        push @$queue, $value;
-        return;
-    }
-
-    my $inner_observable = $observable_factory->($value);
-
-    @$inner_subscriptions = (RxPerl::Subscription->new);
-    $inner_observable->subscribe({
-        new_subscription => $inner_subscriptions->[0],
-        next     => $subscriber->{next},
-        error    => $subscriber->{error},
-        complete => sub {
-            @$inner_subscriptions = ();
-            if (@$queue) {
-                my $new_value = shift @$queue;
-                _op_concat_map_helper(
-                    $new_value, $inner_subscriptions, $observable_factory,
-                    $subscriber, $queue, $own_subscription,
-                );
-            } else {
-                if ($own_subscription->{closed}) {
-                    $subscriber->{complete}->() if defined $subscriber->{complete};
-                }
-            }
-        },
-    });
+        return $source->pipe( op_merge_all(1) );
+    };
 }
 
 sub op_concat_map {
@@ -292,35 +265,10 @@ sub op_concat_map {
     return sub {
         my ($source) = @_;
 
-        return rx_observable->new(sub {
-            my ($subscriber) = @_;
-
-            my @inner_subscriptions;
-            my @queue;
-
-            my $own_subscription = RxPerl::Subscription->new;
-            my $own_subscriber = {
-                new_subscription => $own_subscription,
-                next             => sub {
-                    my ($value) = @_;
-
-                    _op_concat_map_helper(
-                        $value, \@inner_subscriptions, $observable_factory,
-                        $subscriber, \@queue, $own_subscription
-                    );
-                },
-                error            => $subscriber->{error},
-                complete         => sub {
-                    if (! @inner_subscriptions) {
-                        $subscriber->{complete}->() if defined $subscriber->{complete};
-                    }
-                },
-            };
-
-            $source->subscribe($own_subscriber);
-
-            return ($own_subscription, \@inner_subscriptions);
-        });
+        return $source->pipe(
+            op_map($observable_factory),
+            op_concat_all(),
+        );
     };
 }
 
@@ -642,53 +590,73 @@ sub op_every {
     };
 }
 
-sub op_exhaust_map {
-    my ($observable_factory) = @_;
-
+sub op_exhaust_all {
     return sub {
         my ($source) = @_;
 
         return rx_observable->new(sub {
             my ($subscriber) = @_;
 
-            my $inner_subscription;
+            my $active_subscription;
+            my $big_completed;
             my $own_subscription = RxPerl::Subscription->new;
-            my $own_observer = {
+
+            $subscriber->subscription->add(
+                \$active_subscription,
+                $own_subscription,
+            );
+
+            my $own_subscriber = {
                 new_subscription => $own_subscription,
                 next             => sub {
-                    my ($value) = @_;
+                    my ($new_obs) = @_;
 
-                    if (!$inner_subscription or $inner_subscription->{closed}) {
-                        my $inner_observable = $observable_factory->($value);
-                        $inner_subscription = RxPerl::Subscription->new;
-                        my $inner_observer = {
-                            new_subscription => $inner_subscription,
-                            next             => $subscriber->{next},
-                            error            => $subscriber->{error},
-                            complete         => sub {
-                                if ($own_subscription->{closed}) {
-                                    $subscriber->{complete}->() if defined $subscriber->{complete};
-                                }
-                            },
-                        };
-                        $inner_observable->subscribe($inner_observer);
-                    }
+                    !$active_subscription or return;
+                    $active_subscription = RxPerl::Subscription->new;
+                    my $small_subscriber = {
+                        new_subscription => $active_subscription,
+                        next             => sub {
+                            $subscriber->{next}->(@_) if defined $subscriber->{next};
+                        },
+                        error            => sub {
+                            $subscriber->{error}->(@_) if defined $subscriber->{error};
+                        },
+                        complete         => sub {
+                            undef $active_subscription;
+                            $subscriber->{complete}->() if $big_completed and defined $subscriber->{complete};
+                        },
+                    };
+                    $new_obs->subscribe($small_subscriber);
+                },
+                error            => sub {
+                    $subscriber->{error}->(@_) if defined $subscriber->{error};
                 },
-                error            => $subscriber->{error},
                 complete         => sub {
-                    if (! $inner_subscription or $inner_subscription->{closed}) {
-                        $subscriber->{complete}->() if defined $subscriber->{complete};
-                    }
+                    $big_completed = 1;
+                    $subscriber->{complete}->() if !$active_subscription and defined $subscriber->{complete};
                 },
             };
 
-            $source->subscribe($own_observer);
+            $source->subscribe($own_subscriber);
 
-            return ($own_subscription, \$inner_subscription);
+            return;
         });
     };
 }
 
+sub op_exhaust_map {
+    my ($observable_factory) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return $source->pipe(
+            op_map($observable_factory),
+            op_exhaust_all(),
+        );
+    };
+}
+
 sub op_filter {
     my ($filtering_sub) = @_;
 
@@ -822,8 +790,42 @@ sub op_map_to {
     };
 }
 
-sub op_merge_map {
-    my ($observable_factory) = @_;
+sub _op_merge_all_make_subscriber {
+    my ($small_subscriptions, $subscriber, $num_subscriptions_ref, $stored_observables) = @_;
+
+    my $small_subscription = RxPerl::Subscription->new;
+    $small_subscriptions->{$small_subscription} = $small_subscription;
+    return {
+        new_subscription => $small_subscription,
+        next     => sub {
+            $subscriber->{next}->(@_) if defined $subscriber->{next};
+        },
+        error    => sub {
+            $subscriber->{error}->(@_) if defined $subscriber->{error};
+        },
+        complete => sub {
+            $$num_subscriptions_ref--;
+            delete $small_subscriptions->{$small_subscription};
+            if (@$stored_observables) {
+                $$num_subscriptions_ref++;
+                my $new_obs = shift @$stored_observables;
+                $new_obs->subscribe(
+                    _op_merge_all_make_subscriber(
+                        $small_subscriptions,
+                        $subscriber,
+                        $num_subscriptions_ref,
+                        $stored_observables,
+                    ),
+                );
+            } else {
+                $subscriber->{complete}->() if !$$num_subscriptions_ref and defined $subscriber->{complete};
+            }
+        },
+    };
+}
+
+sub op_merge_all {
+    my ($concurrent) = @_;
 
     return sub {
         my ($source) = @_;
@@ -831,44 +833,40 @@ sub op_merge_map {
         return rx_observable->new(sub {
             my ($subscriber) = @_;
 
-            my $subscription = $subscriber->subscription;
+            my $num_subscriptions = 0;
+            my @stored_observables;
+            my %small_subscriptions;
 
-            my %these_subscriptions;
             my $own_subscription = RxPerl::Subscription->new;
-
-            $subscription->add(\%these_subscriptions, $own_subscription);
+            $subscriber->subscription->add(
+                $own_subscription,
+                \%small_subscriptions,
+            );
 
             my $own_subscriber = {
                 new_subscription => $own_subscription,
                 next     => sub {
-                    my ($value) = @_;
+                    my ($new_observable) = @_;
 
-                    my $this_subscription = RxPerl::Subscription->new;
-                    $these_subscriptions{$this_subscription} = $this_subscription;
-
-                    my $this_new_subscriber = {
-                        new_subscription => $this_subscription,
-                        next     => sub {
-                            $subscriber->{next}->(@_) if defined $subscriber->{next};
-                        },
-                        error    => sub {
-                            $subscriber->{error}->(@_) if defined $subscriber->{error};
-                        },
-                        complete => sub {
-                            delete $these_subscriptions{$this_subscription};
-                            $subscriber->{complete}->() if !%these_subscriptions and $own_subscription->{closed}
-                                and defined $subscriber->{complete};
-                        },
-                    };
-
-                    my $this_observable = $observable_factory->($value);
-                    $this_observable->subscribe($this_new_subscriber);
+                    push @stored_observables, $new_observable;
+                    if (! defined $concurrent or $num_subscriptions < $concurrent) {
+                        $num_subscriptions++;
+                        my $new_obs = shift @stored_observables;
+                        $new_obs->subscribe(
+                            _op_merge_all_make_subscriber(
+                                \%small_subscriptions,
+                                $subscriber,
+                                \$num_subscriptions,
+                                \@stored_observables,
+                            ),
+                        );
+                    }
                 },
                 error    => sub {
                     $subscriber->{error}->(@_) if defined $subscriber->{error};
                 },
                 complete => sub {
-                    $subscriber->{complete}->() if !%these_subscriptions and defined $subscriber->{complete};
+                    $subscriber->{complete}->() if !$num_subscriptions and defined $subscriber->{complete};
                 },
             };
 
@@ -879,6 +877,19 @@ sub op_merge_map {
     };
 }
 
+sub op_merge_map {
+    my ($observable_factory) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return $source->pipe(
+            op_map($observable_factory),
+            op_merge_all(),
+        );
+    };
+}
+
 sub op_merge_with {
     my @other_sources = @_;
 
@@ -1384,53 +1395,49 @@ sub op_start_with {
     };
 }
 
-sub op_switch_map {
-    my ($observable_factory) = @_;
-
+sub op_switch_all {
     return sub {
         my ($source) = @_;
 
         return rx_observable->new(sub {
             my ($subscriber) = @_;
 
-            my $this_own_subscription;
-            my $own_subscription = RxPerl::Subscription->new;
+            my $old_subscription;
 
-            $subscriber->subscription->add(
-                \$this_own_subscription, $own_subscription,
-            );
+            my $chief_is_complete;
+            my $sub_is_complete;
 
-            my $own_subscriber = {
-                new_subscription => $own_subscription,
+            my $obs_subscriber = {
                 next     => sub {
-                    my ($value) = @_;
-
-                    my $new_observable = $observable_factory->($value);
-                    $this_own_subscription->unsubscribe() if $this_own_subscription;
-                    $this_own_subscription = RxPerl::Subscription->new;
-                    my $this_own_subscriber = {
-                        new_subscription => $this_own_subscription,
-                        next     => sub {
-                            $subscriber->{next}->(@_) if defined $subscriber->{next};
-                        },
-                        error    => sub {
-                            $subscriber->{error}->(@_) if defined $subscriber->{error};
-                        },
-                        complete => sub {
-                            $subscriber->{complete}->() if $own_subscription->{closed}
-                                and defined $subscriber->{complete};
-                        },
-                    };
-
-                    $new_observable->subscribe($this_own_subscriber);
+                    $subscriber->{next}->(@_) if defined $subscriber->{next};
                 },
                 error    => sub {
                     $subscriber->{error}->(@_) if defined $subscriber->{error};
                 },
                 complete => sub {
-                    # $source_complete = 1;
-                    $subscriber->{complete}->() if defined $subscriber->{complete} and
-                        (not $this_own_subscription or $this_own_subscription->{closed});
+                    $sub_is_complete = 1;
+                    $subscriber->{complete}->() if $chief_is_complete and defined $subscriber->{complete};
+                },
+            };
+
+            my $own_subscription = RxPerl::Subscription->new;
+            $subscriber->subscription->add(\$old_subscription, $own_subscription);
+
+            my $own_subscriber = {
+                new_subscription => $own_subscription,
+                next     => sub {
+                    my ($new_observable) = @_;
+
+                    $sub_is_complete = 0;
+                    $old_subscription->unsubscribe() if $old_subscription;
+                    $old_subscription = $new_observable->subscribe($obs_subscriber);
+                },
+                error    => sub {
+                    $subscriber->{error}->(@_) if defined $subscriber->{error};
+                },
+                complete => sub {
+                    $chief_is_complete = 1;
+                    $subscriber->{complete}->() if $sub_is_complete and defined $subscriber->{complete};
                 },
             };
 
@@ -1438,6 +1445,19 @@ sub op_switch_map {
 
             return;
         });
+    };
+}
+
+sub op_switch_map {
+    my ($observable_factory) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return $source->pipe(
+            op_map($observable_factory),
+            op_switch_all(),
+        );
     }
 }
 
